@@ -62,7 +62,7 @@ rayDISC<-function(phy,data, ntraits=1, charnum=1, rate.mat=NULL, model=c("ER","S
 	#Some initial values for use later - will clean up
 	k <- 1 # Only one trait allowed
 	factored <- factorData(workingData,charnum=charnum) # just factoring to figure out how many levels (i.e. number of states) in data.
-	nl <- ncol(factored)
+    nl <- ncol(factored)
 	state.names <- colnames(factored) # for subsequent reporting
 	bound.hit <- FALSE # to keep track of whether min.rate is one of the rate estimates (and thus, potentially a non-optimal rate)
 	# Check to make sure values are reasonable (i.e. non-negative)
@@ -85,7 +85,7 @@ rayDISC<-function(phy,data, ntraits=1, charnum=1, rate.mat=NULL, model=c("ER","S
 	root.p=root.p	
 	ip=ip
 
-	model.set.final<-rate.cat.set.oneT(phy=phy,data=workingData,model=model,charnum=charnum)
+	model.set.final<-rate.cat.set.rayDISC(phy=phy,data=workingData,model=model,charnum=charnum)
 	if(!is.null(rate.mat)){
 		rate <- rate.mat
 		model.set.final$np <- max(rate, na.rm=TRUE)
@@ -110,7 +110,7 @@ rayDISC<-function(phy,data, ntraits=1, charnum=1, rate.mat=NULL, model=c("ER","S
 			cat("Initializing...", "\n")
 			#Sets parameter settings for random restarts by taking the parsimony score and dividing
 			#by the total length of the tree
-			model.set.init<-rate.cat.set.oneT(phy=phy,data=workingData,model="ER",charnum=charnum)
+			model.set.init<-rate.cat.set.rayDISC(phy=phy,data=workingData,model="ER",charnum=charnum)
 			opts <- list("algorithm"="NLOPT_LN_SBPLX", "maxeval"="1000000", "ftol_rel"=.Machine$double.eps^0.5)
 			dat<-as.matrix(workingData)
 			dat<-phyDat(dat,type="USER", levels=levels(as.factor(workingData[,1])))
@@ -181,13 +181,12 @@ rayDISC<-function(phy,data, ntraits=1, charnum=1, rate.mat=NULL, model=c("ER","S
 	if((any(solution == lb,na.rm = TRUE) || any(solution == ub,na.rm = TRUE)) && (lb != 0 || ub != 100)){
 		bound.hit <- TRUE
 	}
-
 	rownames(solution) <- rownames(solution.se) <- state.names
 	colnames(solution) <- colnames(solution.se) <- state.names
 
 	if(is.character(node.states)){
 		if (node.states == "marginal" || node.states == "scaled"){
-			colnames(lik.anc$lik.anc.states) <- state.names
+            colnames(lik.anc$lik.anc.states) <- state.names
 		}
 	}
 	obj = list(loglik = loglik, AIC = -2*loglik+2*model.set.final$np,AICc = -2*loglik+(2*model.set.final$np*(nb.tip/(nb.tip-model.set.final$np-1))),ntraits=1, solution=solution, solution.se=solution.se, index.mat=model.set.final$index.matrix, opts=opts, data=data, phy=phy, states=lik.anc$lik.anc.states, tip.states=tip.states, iterations=out$iterations, eigval=eigval, eigvect=eigvect,bound.hit=bound.hit) 
@@ -275,40 +274,50 @@ dev.raydisc<-function(p,phy,liks,Q,rate,root.p){
 	#If any of the logs have NAs restart search:
 	if (is.na(sum(log(comp[-TIPS])))){return(1000000)}
 	else{
-		equil.root <- NULL
-		for(i in 1:ncol(Q)){
-			posrows <- which(Q[,i] >= 0)
-			rowsum <- sum(Q[posrows,i])
-			poscols <- which(Q[i,] >= 0)
-			colsum <- sum(Q[i,poscols])
-			equil.root <- c(equil.root,rowsum/(rowsum+colsum))
-		}		
-		if (is.null(root.p)){
-			flat.root = equil.root
-			k.rates <- 1/length(which(!is.na(equil.root)))
-			flat.root[!is.na(flat.root)] = k.rates
-			flat.root[is.na(flat.root)] = 0
-			loglik<- -(sum(log(comp[-TIPS])) + log(sum(flat.root * liks[root,])))
-		}
-		else{
-			#root.p==maddfitz will fix root probabilities according to FitzJohn et al 2009 Eq. 10:
-			if(is.character(root.p)){
-				equil.root[is.na(equil.root)] = 0
-				loglik <- -(sum(log(comp[-TIPS])) + log(sum(equil.root * liks[root,])))
-				if(is.infinite(loglik)){return(1000000)}
-			}
-			#root.p!==NULL will fix root probabilities based on user supplied vector:
-			else{
-				loglik<- -(sum(log(comp[-TIPS])) + log(sum(root.p * liks[root,])))
-				if(is.infinite(loglik)){return(1000000)}
-			}
-		}
+        equil.root <- NULL
+        for(i in 1:ncol(Q)){
+            posrows <- which(Q[,i] >= 0)
+            rowsum <- sum(Q[posrows,i])
+            poscols <- which(Q[i,] >= 0)
+            colsum <- sum(Q[i,poscols])
+            equil.root <- c(equil.root,rowsum/(rowsum+colsum))
+        }
+        if (is.null(root.p)){
+            flat.root = equil.root
+            k.rates <- 1/length(which(!is.na(equil.root)))
+            flat.root[!is.na(flat.root)] = k.rates
+            flat.root[is.na(flat.root)] = 0
+            loglik<- -(sum(log(comp[-TIPS])) + log(sum(flat.root * liks[root,])))
+        }
+        else{
+            if(is.character(root.p)){
+                # root.p==yang will fix root probabilities based on the inferred rates: q10/(q01+q10)
+                if(root.p == "yang"){
+                    diag(Q) = 0
+                    equil.root = colSums(Q) / sum(Q)
+                    loglik <- -(sum(log(comp[-TIPS])) + log(sum(exp(log(equil.root)+log(liks[root,])))))
+                    if(is.infinite(loglik)){
+                        return(1000000)
+                    }
+                }else{
+                    # root.p==maddfitz will fix root probabilities according to FitzJohn et al 2009 Eq. 10:
+                    root.p = liks[root,] / sum(liks[root,])
+                    loglik <- -(sum(log(comp[-TIPS])) + log(sum(exp(log(root.p)+log(liks[root,])))))
+                }
+            }
+            # root.p!==NULL will fix root probabilities based on user supplied vector:
+            else{
+                loglik <- -(sum(log(comp[-TIPS])) + log(sum(exp(log(root.p)+log(liks[root,])))))
+                if(is.infinite(loglik)){
+                    return(1000000)
+                }
+            }
+        }
 	}
 	loglik
 }
 
-rate.cat.set.oneT<-function(phy,data,model,charnum){
-	
+rate.cat.set.rayDISC<-function(phy,data,model,charnum){
 	k <- 1
 	factored <- factorData(data, charnum=charnum)
 	nl <- ncol(factored)
@@ -340,7 +349,6 @@ rate.cat.set.oneT<-function(phy,data,model,charnum){
 	obj$Q<-Q
 
 	return(obj)
-
 }
 
 #########################
@@ -446,7 +454,7 @@ factorData <- function(data,whichchar=1,charnum){
 		ampLocs <- findAmps(levelstring, charnum)
 		if(length(ampLocs) == 0){ #No ampersands, character is monomorphic
 			currlvl <- levelstring
-			if(currlvl == "?" || currlvl == "-"){ # Check for missing data
+			if(currlvl == "?" || currlvl == "-" || currlvl == "NA"){ # Check for missing data
 				missing <- c(missing,row) # add to list of taxa with missing values, will fill in entire row later
 			}
 			else { # Not missing data
